@@ -34,8 +34,8 @@ GPL.
 // enigmailCommon.js: shared JS functions for Enigmail
 
 // This Enigmail version and compatible Enigmime version
-var gEnigmailVersion = "0.84.2.0";
-var gEnigmimeVersion = "0.84.2.0";
+var gEnigmailVersion = "0.86.1.0";
+var gEnigmimeVersion = "0.86.1.0";
 
 // Maximum size of message directly processed by Enigmail
 const ENIG_MSG_BUFFER_SIZE = 96000;
@@ -60,6 +60,8 @@ const ENIG_ASS_CONTRACTID = "@mozilla.org/appshell/appShellService;1";
 const ENIG_CLIPBOARD_CONTRACTID = "@mozilla.org/widget/clipboard;1";
 const ENIG_CLIPBOARD_HELPER_CONTRACTID = "@mozilla.org/widget/clipboardhelper;1"
 const ENIG_TRANSFERABLE_CONTRACTID = "@mozilla.org/widget/transferable;1"
+const ENIG_LOCALE_SVC_CONTRACTID = "@mozilla.org/intl/nslocaleservice;1";
+const ENIG_DATE_FORMAT_CONTRACTID = "@mozilla.org/intl/scriptabledateformat;1"
 
 const ENIG_LOCALFILEOUTPUTSTREAM_CONTRACTID =
                               "@mozilla.org/network/file-output-stream;1";
@@ -92,6 +94,8 @@ const PGP_MIME_NEVER    = 0;
 const PGP_MIME_POSSIBLE = 1;
 const PGP_MIME_ALWAYS   = 2;
 
+const ENIG_POSSIBLE_PGPMIME = -2081;
+
 // property name for temporary directory service
 const ENIG_TEMPDIR_PROP = "TmpD";
 
@@ -115,48 +119,6 @@ const ENIG_THREE_BUTTON_STRINGS   = (ENIG_BUTTON_TITLE_IS_STRING * ENIG_BUTTON_P
                                (ENIG_BUTTON_TITLE_IS_STRING * ENIG_BUTTON_POS_1) +
                                (ENIG_BUTTON_TITLE_IS_STRING * ENIG_BUTTON_POS_2);
 
-/* moved to enigprefs.js!
-var gEnigmailPrefDefaults = {"configuredVersion":"",
-                             "agentPath":"",
-                             "alwaysTrustSend":true,
-                             "autoCrypto":false,
-                             "autoDecrypt":true,
-                             "captureWebMail":false,
-                             "composeHtmlAlertCount":3,
-                             "confirmBeforeSend":false,
-                             "disableSMIMEui":false,
-                             "displaySignWarn":true,
-                             "displayPartiallySigned":true,
-                             "doubleDashSeparator":true,
-                             "encryptAttachments":1,
-                             "encryptToSelf":true,
-                             "gpgVersionWarnCount":1,
-                             "handleDoubleClick":false,
-                             "hushMailSupport":false,
-                             "initAlertCount":2,
-                             "inlineAttachAsciiArmor":false,
-                             "inlineAttachExt":".pgp",
-                             "logDirectory":"",
-                             "keepSettingsForReply":true,
-                             "keyserver":"random.sks.keyserver.penguin.de, pgp.dtype.org, keyserver.kjsl.com, ldap://certserver.pgp.com",
-                             "maxIdleMinutes":5,
-                             "mimeHashAlgorithm":1,
-                             "noPassphrase":false,
-                             "parseAllHeaders":true,
-                             "quotedPrintableWarn":0,
-                             "respectHttpProxy":true,
-                             "recipientsSelectionOption":1,
-                             "saveEncrypted":0,
-                             "show_headers":1,
-                             "supportMultiPass":false,
-                             "useDefaultComment":false,
-                             "useMimeExperimental":false,
-                             "useGpgAgent":false,
-                             "usePGPMimeOption":PGP_MIME_POSSIBLE,
-                             "wrapHtmlBeforeSend":true
-                            };
-*/
-
 var gEnigLogLevel = 2;     // Output only errors/warnings by default
 var gEnigDebugLog;
 
@@ -536,6 +498,25 @@ function EnigAlertCount(countPrefName, mesg) {
   EnigAlert(mesg);
 }
 
+function EnigAlertPref(mesg, prefText) {
+  const display = true;
+  const dontDisplay = false;
+
+  var prefValue = EnigGetPref(prefText);
+  if (prefValue == display) {
+    var checkBoxObj = { value: false } ;
+    var buttonPressed = gEnigPromptSvc.confirmEx(window,
+                          EnigGetString("enigAlert"),
+                          mesg,
+                          (gEnigPromptSvc.BUTTON_TITLE_OK * ENIG_BUTTON_POS_0),
+                          null, null, null,
+                          EnigGetString("dlgNoPrompt"), checkBoxObj);
+    if (checkBoxObj.value && buttonPressed==0) {
+      EnigSetPref(prefText, dontDisplay);
+    }
+  }
+}
+
 function EnigConfirm(mesg) {
   var dummy=new Object();
 
@@ -636,6 +617,11 @@ function EnigHelpWindow(source) {
 }
 
 function EnigUpgrade() {
+  var ioService = Components.classes[ENIG_IOSERVICE_CONTRACTID].getService(Components.interfaces.nsIIOService);
+  if (ioService && ioService.offline) {
+    EnigAlert(EnigGetString("needOnline"));
+    return;
+  }
   window.openDialog("http://enigmail.mozdev.org/no_wrap/update.html?upgrade=yes&enigmail="+gEnigmailVersion+"&enigmime="+gEnigmimeVersion, "dialog");
 }
 
@@ -854,6 +840,8 @@ function EnigConvertFromUnicode(text, charset) {
     return unicodeConv.ConvertFromUnicode(text);
 
   } catch (ex) {
+    DEBUG_LOG("enigmailCommon.js: EnigConvertFromUnicode: caught an exception\n");
+  
     return text;
   }
 }
@@ -873,6 +861,7 @@ function EnigConvertToUnicode(text, charset) {
     return unicodeConv.ConvertToUnicode(text);
 
   } catch (ex) {
+    DEBUG_LOG("enigmailCommon.js: EnigConvertToUnicode: caught an exception\n");
     return text;
   }
 }
@@ -1028,6 +1017,7 @@ function EnigClearPassphrase() {
     return;
 
   enigmailSvc.clearCachedPassphrase();
+  EnigAlertPref(EnigGetString("passphraseCleared"), "warnClearPassphrase");
 }
 
 function EnigOpenWin (winName, spec, winOptions, optList) {
@@ -1067,7 +1057,7 @@ function EnigViewConsole() {
   DEBUG_LOG("enigmailCommon.js: EnigViewConsole\n");
 
   EnigOpenWin("enigmail:console",
-              "chrome://enigmail/content/enigmailConsole.xul");
+              "chrome://enigmail/content/enigmailConsole.xul", "chrome,resizable");
 }
 
 function EnigViewDebugLog() {
@@ -1108,7 +1098,7 @@ function EnigKeygen() {
   DEBUG_LOG("enigmailCommon.js: EnigKeygen\n");
 
   window.openDialog('chrome://enigmail/content/enigmailKeygen.xul',
-                    'Enigmail Key Generation',
+                    EnigGetString('keyGeneration'),
                     'chrome,dialog,modal,close=no,resizable=yes,width=600');
 
 }
@@ -1433,3 +1423,11 @@ function EnigNewRule(emailAddress) {
   return true;
 }
 
+function EnigSetKeyTrust(parent, userId, keyId, trustLevel, exitCodeObj, errorMsgObj) {
+
+  var enigmailSvc = GetEnigmailSvc();
+  if (!enigmailSvc)
+    return false;
+  var r = enigmailSvc.setKeyTrust(parent, userId, keyId, trustLevel, exitCodeObj, errorMsgObj);
+  return r;
+}
\ No newline at end of file
