@@ -467,11 +467,18 @@ function enigMessageImport(event) {
   return enigMessageParse(!event, true, "");
 }
 
-function enigMessageDecrypt(event) {
+// callback function for automatic decryption
+function enigMessageAutoDecrypt(event) {
+  enigMessageDecrypt(event, true);
+}
+
+// analyse message header and decrypt/verify message
+function enigMessageDecrypt(event, isAuto) {
   DEBUG_LOG("enigmailMessengerOverlay.js: enigMessageDecrypt: "+event+"\n");
 
   if (EnigGetPref("parseAllHeaders")) {
-    var showHeaders = 1;
+    var showHeaders = 0;
+    
     try {
       showHeaders = gEnigPrefRoot.getIntPref("mail.show_headers");
     } catch (ex) {
@@ -524,9 +531,7 @@ function enigMessageDecrypt(event) {
     }
 
   }
-
-  EnigShowHeadersAll(false);
-
+  
   var contentType = "";
   var contentEncoding = "";
   var xEnigmailVersion = "";
@@ -537,16 +542,33 @@ function enigMessageDecrypt(event) {
     xEnigmailVersion = gEnigSavedHeaders["x-enigmail-version"];
   }
 
-  if ((contentType.search(/^multipart\/encrypted(;|$)/i) == 0) ||
-      (embeddedEncrypted && contentType.search(/^multipart\/mixed(;|$)/i) == 0)
-      /* && (!embeddedSigned) */ ) {
+  var enigmailSvc = GetEnigmailSvc();
+  if (!enigmailSvc)
+    return;
+    
+  if (isAuto && (! EnigGetPref("autoDecrypt"))) {
+    var signedMsg = ((contentType.search(/^multipart\/signed(;|$)/i) == 0) && (contentType.search(/application\/pgp-signature/i)>0));
+    var encrypedMsg = ((contentType.search(/^multipart\/encrypted(;|$)/i) == 0) && (contentType.search(/application\/pgp-encrypted/i)>0));
+    if (embeddedSigned || embeddedEncrypted || 
+        encrypedMsg || signedMsg) {
+  
+      if (
+          (!enigmailSvc.mimeInitialized() && encrypedMsg) || signedMsg ||
+          ((!encrypedMsg) && (embeddedSigned || embeddedEncrypted))) {
+        enigUpdateHdrIcons(ENIG_POSSIBLE_PGPMIME, 0, "", "", "", EnigGetString("possiblyPgpMime"));
+      }
+    }
+    return;
+  }
+
+  EnigShowHeadersAll(false);
+
+  if (((contentType.search(/^multipart\/encrypted(;|$)/i) == 0) ||
+      (embeddedEncrypted && contentType.search(/^multipart\/mixed(;|$)/i) == 0))
+       && (!embeddedSigned)) {
     // multipart/encrypted
     DEBUG_LOG("enigmailMessengerOverlay.js: multipart/encrypted\n");
 
-    var enigmailSvc = GetEnigmailSvc();
-    if (!enigmailSvc)
-      return;
-
     if (!enigmailSvc.mimeInitialized()) {
       // Display enigmail:dummy URL in message pane to initialize
 
@@ -566,12 +588,16 @@ function enigMessageDecrypt(event) {
   }
 
   var tryVerify = false;
+  var msgUriSpec = enigGetCurrentMsgUriSpec();
   var enableSubpartTreatment = false;
   // special treatment for embedded signed messages
   if (embeddedSigned) {
-    if (contentType.search(/^multipart\/encrypted(;|$)/i) == 0)
-      tryVerify = true;
-
+    if (contentType.search(/^multipart\/encrypted(;|$)/i) == 0) {
+      var msgSize=messenger.messageServiceFromURI(msgUriSpec).messageURIToMsgHdr(msgUriSpec).messageSize
+      // workaround for bug 5777: try to verify if msg size < 40000
+      // to avoid blocking Mozilla
+      if (msgSize < 40000) tryVerify = true;
+    }
     if (contentType.search(/^multipart\/mixed(;|$)/i) == 0) {
       tryVerify = true;
       enableSubpartTreatment = true;
@@ -579,9 +605,9 @@ function enigMessageDecrypt(event) {
   }
 
   if ((contentType.search(/^multipart\/signed(;|$)/i) == 0) &&
-       (contentType.search(/application\/pgp-signature/i) >= 0))
+       (contentType.search(/application\/pgp-signature/i) >= 0)) {
     tryVerify=true;
-
+  }
   if (tryVerify) {
     // multipart/signed
     DEBUG_LOG("enigmailMessengerOverlay.js: multipart/signed\n");
@@ -590,7 +616,6 @@ function enigMessageDecrypt(event) {
     if (!enigmailSvc)
       return;
 
-    var msgUriSpec = enigGetCurrentMsgUriSpec();
     var mailNewsUrl = enigGetCurrentMsgUrl();
     if (embeddedSigned) {
       mailNewsUrl.spec = embeddedSigned;
@@ -713,6 +738,7 @@ function enigMessageParseCallback(msgText, contentEncoding, charset, interactive
     var exitCodeObj    = new Object();
     var statusFlagsObj = new Object();
     var userIdObj      = new Object();
+    var sigDetailsObj  = new Object();
 
     var signatureObj = new Object();
     signatureObj.value = signature;
@@ -724,7 +750,7 @@ function enigMessageParseCallback(msgText, contentEncoding, charset, interactive
 
     plainText = enigmailSvc.decryptMessage(window, uiFlags, msgText,
                                  signatureObj, exitCodeObj, statusFlagsObj,
-                                 keyIdObj, userIdObj, errorMsgObj);
+                                 keyIdObj, userIdObj, sigDetailsObj, errorMsgObj);
 
     //DEBUG_LOG("enigmailMessengerOverlay.js: enigMessageParseCallback: plainText='"+plainText+"'\n");
 
@@ -744,7 +770,7 @@ function enigMessageParseCallback(msgText, contentEncoding, charset, interactive
      return;
   }
 
-  enigUpdateHdrIcons(exitCode, statusFlags, keyIdObj.value, userIdObj.value, errorMsg);
+  enigUpdateHdrIcons(exitCode, statusFlags, keyIdObj.value, userIdObj.value, sigDetailsObj.value, errorMsg);
 
   if (statusFlags & (nsIEnigmail.BAD_SIGNATURE | nsIEnigmail.BAD_ARMOR)) {
     // Bad signature/armor
@@ -825,7 +851,6 @@ function enigMessageParseCallback(msgText, contentEncoding, charset, interactive
 
   var msgFrame = EnigGetFrame(window, "messagepane");
   var bodyElement = msgFrame.document.getElementsByTagName("body")[0];
-
   try {
     // Create and load one-time message URI
     var messageContent = enigGetDecryptedMessage("message/rfc822", false);
@@ -838,11 +863,12 @@ function enigMessageParseCallback(msgText, contentEncoding, charset, interactive
                                            messageContent,
                                            false);
     gEnigCreatedURIs.push(uri);
-
     msgFrame.location = uri;
 
+
   } catch (ex) {
     // Display plain text with hyperlinks
+    DEBUG_LOG("enigmailMessengerOverlay.js: enigMessageParseCallback: caught exception\n");
 
     // Get selection range for inserting HTML
     var domSelection = msgFrame._content.getSelection();
@@ -900,8 +926,8 @@ function enigMessageParseCallback(msgText, contentEncoding, charset, interactive
 
 // check if the attachment could be encrypted
 function enigCheckEncryptedAttach(attachment) {
-  return (attachment.displayName.match(/\.(gpg|pgp|asc)$/) ||
-      attachment.contentType.match(/^application\/pgp(\-.*)?$/));
+  return (attachment.displayName.match(/\.(gpg|pgp|asc)$/i) ||
+      attachment.contentType.match(/^application\/pgp(\-.*)?$/i));
 }
 
 function enigEscapeTextForHTML(text, hyperlink) {
@@ -1484,76 +1510,14 @@ function enigHandleAttachmentSel(actionType) {
 
 function enigHandleAttachment(actionType, anAttachment) {
   DEBUG_LOG("enigmailMessengerOverlay.js: enigHandleAttachment: actionType="+actionType+", anAttachment(url)="+anAttachment.url+"\n");
-
+ 
   var ipcBuffer = Components.classes[ENIG_IPCBUFFER_CONTRACTID].createInstance(Components.interfaces.nsIIPCBuffer);
 
-
-  var callbackArg = { actionType:actionType,
-                      attachment:anAttachment,
-                      forceBrowser:false,
-                      ipcBuffer:ipcBuffer
-                    };
-
-  var requestObserver = new EnigRequestObserver(enigHandleAttachmentCallback,
-                                                callbackArg);
-
-  var ioServ = Components.classes[ENIG_IOSERVICE_CONTRACTID].getService(Components.interfaces.nsIIOService);
-
-  ipcBuffer.open(100, false); // get just the first bit
-  var msgUri = ioServ.newURI(anAttachment.url, null, null);
-
-  ipcBuffer.observe(requestObserver, msgUri);
-
-  var channel = ioServ.newChannelFromURI(msgUri);
-
-  var pipeFilter = Components.classes[ENIG_PIPEFILTERLISTENER_CONTRACTID].createInstance(Components.interfaces.nsIPipeFilterListener);
-
-  pipeFilter.init(ipcBuffer, null,
-                "",
-                "",
-                0, false, false, null);
-
-  var listener;
-  listener = pipeFilter;
-
-  channel.asyncOpen(listener, msgUri);
-}
-
-function enigHandleAttachmentCallback(callbackArg, ctxt) {
-  DEBUG_LOG("enigmailMessengerOverlay.js: enigHandleAttachmentCallback: "+ctxt+"\n");
-
-  if (callbackArg.ipcBuffer.overflowed) {
-    WARNING_LOG("enigmailMessengerOverlay.js: enigHandleAttachmentCallback: MESSAGE BUFFER OVERFLOW\n");
-  }
-
-  var dataLengthObj = new Object();
-  var attachmentHead = callbackArg.ipcBuffer.getByteData(dataLengthObj);
-  callbackArg.ipcBuffer.shutdown();
-
-
-  if (attachmentHead.match(/\-\-\-\-\-BEGIN PGP \w+ KEY BLOCK\-\-\-\-\-/)) {
-    // attachment appears to be a PGP key file
-
-    if (EnigConfirm(EnigGetString("attachmentPgpKey", callbackArg.attachment.displayName))) {
-      enigImportAttachment(callbackArg);
-    }
-    else {
-      EnigLoadURLInNavigatorWindow(callbackArg.attachment.url, true)
-    }
-  }
-  else {
-    // attachment is probably encrypted
-    enigDecryptAttachment(callbackArg);
-  }
-}
-
-// try to decrypt an attachment
-function enigDecryptAttachment(argumentsObj) {
-  DEBUG_LOG("enigmailMessengerOverlay.js: enigDecryptAttachment: url"+argumentsObj.attachment.url+"\n");
-
-  var ipcBuffer = Components.classes[ENIG_IPCBUFFER_CONTRACTID].createInstance(Components.interfaces.nsIIPCBuffer);
-
-  argumentsObj.ipcBuffer=ipcBuffer;
+  var argumentsObj = { actionType: actionType,
+                       attachment: anAttachment,
+                       forceBrowser: false,
+                       ipcBuffer: ipcBuffer
+                     };
 
   var requestObserver = new EnigRequestObserver(enigDecryptAttachmentCallback,
                                                 argumentsObj);
@@ -1593,7 +1557,7 @@ function enigDecryptAttachmentCallback(callbackArg, ctxt) {
   var errorMsgObj= new Object();
   var enigmailSvc =  GetEnigmailSvc();
   var outFile;
-  var rawFileName=callbackArg.attachment.displayName.replace(/\.(asc|pgp|gpg)$/,"");
+  var rawFileName=callbackArg.attachment.displayName.replace(/\.(asc|pgp|gpg)$/i,"");
 
   if (callbackArg.actionType == "saveAttachment") {
     outFile = EnigFilePicker(EnigGetString("saveAttachmentHeader"),
@@ -1620,7 +1584,9 @@ function enigDecryptAttachmentCallback(callbackArg, ctxt) {
     }
   }
 
-  var exitStatus=enigmailSvc.decryptAttachment(window, outFile.path, callbackArg.ipcBuffer,
+  var exitStatus=enigmailSvc.decryptAttachment(window, outFile.path,
+                                callbackArg.attachment.displayName, 
+                                callbackArg.ipcBuffer,
                                 exitCodeObj, statusFlagsObj,
                                 errorMsgObj);
 
@@ -1641,57 +1607,65 @@ function enigDecryptAttachmentCallback(callbackArg, ctxt) {
       exiStatus=false;
     }
   }
-  if (exitStatus && callbackArg.actionType == "openAttachment") {
-    var ioServ = Components.classes[ENIG_IOSERVICE_CONTRACTID].getService(Components.interfaces.nsIIOService);
-    var outFileUri = ioServ.newFileURI(outFile);
-    var fileExt = outFile.leafName.replace(/(.*\.)(\w+)$/, "$2")
-    if (fileExt && ! callbackArg.forceBrowser) {
-      try {
-        var extAppLauncher = Components.classes[ENIG_MIME_CONTRACTID].getService(Components.interfaces.nsPIExternalAppLauncher);
-        extAppLauncher.deleteTemporaryFileOnExit(outFile);
-
-        var mimeService = Components.classes[ENIG_MIME_CONTRACTID].getService(Components.interfaces.nsIMIMEService);
-        var fileMimeType = mimeService.GetTypeFromFile(outFile);
-        var fileMimeInfo = mimeService.GetFromTypeAndExtension(fileMimeType, fileExt);
-        if (gEnigDecryptButton && gEnigDecryptButton.getAttribute("buttontype")=="seamonkey") {
-          // Seamonkey variant
-          if ((fileMimeInfo.preferredAction == fileMimeInfo.useHelperApp ||
-              fileMimeInfo.preferredAction == fileMimeInfo.useSystemDefault) &&
-              (! fileMimeType.indexOf("text/")==0)) {
-            // open attachment with a helper application
-            window.OpenURL(outFileUri.asciiSpec);
-            return;
-          }
-        }
-        else {
-          // Thunderbird variant
-          if (fileMimeType.indexOf("text/")==0 || fileMimeType.indexOf("image/")==0) {
-            // open attachment using a browser window
-            var w = window.open(outFileUri.asciiSpec,
-                        "_blank",
-                        "chrome,extrachrome,menubar,resizable,scrollbars,status,toolbar,width=600,height=400");
-            w.title=outFile.leafName;
-            if (fileMimeType.indexOf("text/")==0)
-                w.document.designMode="on";
-            return;
+  if (exitStatus) {
+    if (statusFlagsObj.value & nsIEnigmail.IMPORTED_KEY) {
+      EnigAlert(EnigGetString("successKeyImport")+"\n\n"+errorMsgObj.value);
+    }
+    else if (statusFlagsObj.value & nsIEnigmail.DISPLAY_MESSAGE) {
+      EnigLoadURLInNavigatorWindow(callbackArg.attachment.url, true)
+    }
+    else if (callbackArg.actionType == "openAttachment") {
+      var ioServ = Components.classes[ENIG_IOSERVICE_CONTRACTID].getService(Components.interfaces.nsIIOService);
+      var outFileUri = ioServ.newFileURI(outFile);
+      var fileExt = outFile.leafName.replace(/(.*\.)(\w+)$/, "$2")
+      if (fileExt && ! callbackArg.forceBrowser) {
+        try {
+          var extAppLauncher = Components.classes[ENIG_MIME_CONTRACTID].getService(Components.interfaces.nsPIExternalAppLauncher);
+          extAppLauncher.deleteTemporaryFileOnExit(outFile);
+  
+          var mimeService = Components.classes[ENIG_MIME_CONTRACTID].getService(Components.interfaces.nsIMIMEService);
+          var fileMimeType = mimeService.GetTypeFromFile(outFile);
+          var fileMimeInfo = mimeService.GetFromTypeAndExtension(fileMimeType, fileExt);
+          if (gEnigDecryptButton && gEnigDecryptButton.getAttribute("buttontype")=="seamonkey") {
+            // Seamonkey variant
+            if ((fileMimeInfo.preferredAction == fileMimeInfo.useHelperApp ||
+                fileMimeInfo.preferredAction == fileMimeInfo.useSystemDefault) &&
+                (! fileMimeType.indexOf("text/")==0)) {
+              // open attachment with a helper application
+              window.OpenURL(outFileUri.asciiSpec);
+              return;
+            }
           }
           else {
-            // open attachment with a helper application
-            messenger.OpenURL(outFileUri.asciiSpec);
-            return;
+            // Thunderbird variant
+            if (fileMimeType.indexOf("text/")==0 || fileMimeType.indexOf("image/")==0) {
+              // open attachment using a browser window
+              var w = window.open(outFileUri.asciiSpec,
+                          "_blank",
+                          "chrome,extrachrome,menubar,resizable,scrollbars,status,toolbar,width=600,height=400");
+              w.title=outFile.leafName;
+              if (fileMimeType.indexOf("text/")==0)
+                  w.document.designMode="on";
+              return;
+            }
+            else {
+              // open attachment with a helper application
+              messenger.OpenURL(outFileUri.asciiSpec);
+              return;
+            }
           }
         }
+        catch (ex) {
+          // if the attachment file type is unknown, an exception is thrown,
+          // so let it be handled by a browser window
+          enigLoadExternalURL(outFileUri.asciiSpec);
+          return;
+        }
       }
-      catch (ex) {
-        // if the attachment file type is unknown, an exception is thrown,
-        // so let it be handled by a browser window
-        enigLoadExternalURL(outFileUri.asciiSpec);
-        return;
-      }
+  
+      // open the attachment using an external application
+      enigLoadExternalURL(outFileUri.asciiSpec);
     }
-
-    // open the attachment using an external application
-    enigLoadExternalURL(outFileUri.asciiSpec);
   }
 }
 
@@ -1704,63 +1678,6 @@ function enigLoadExternalURL(url) {
   }
 }
 
-// try to import the PGP keys contained in an attachment
-function enigImportAttachment(argumentsObj) {
-  DEBUG_LOG("enigmailMessengerOverlay.js: enigImportAttachment: url"+argumentsObj.attachment.url+"\n");
-
-  var ipcBuffer = Components.classes[ENIG_IPCBUFFER_CONTRACTID].createInstance(Components.interfaces.nsIIPCBuffer);
-
-  argumentsObj.ipcBuffer=ipcBuffer;
-
-  var requestObserver = new EnigRequestObserver(enigImportAttachmentCallback,
-                                                argumentsObj);
-
-  var ioServ = Components.classes[ENIG_IOSERVICE_CONTRACTID].getService(Components.interfaces.nsIIOService);
-
-  ipcBuffer.open(99999999999, false); // -1 doesn't work, so I set it to a _big_ number ...
-  var msgUri = ioServ.newURI(argumentsObj.attachment.url, null, null);
-
-  ipcBuffer.observe(requestObserver, msgUri);
-
-  var channel = ioServ.newChannelFromURI(msgUri);
-
-  var pipeFilter = Components.classes[ENIG_PIPEFILTERLISTENER_CONTRACTID].createInstance(Components.interfaces.nsIPipeFilterListener);
-
-  pipeFilter.init(ipcBuffer, null,
-                "",
-                "",
-                0, false, false, null);
-
-  var listener;
-  listener = pipeFilter;
-
-  channel.asyncOpen(listener, msgUri);
-}
-
-
-function enigImportAttachmentCallback(callbackArg, ctxt) {
-  DEBUG_LOG("enigmailMessengerOverlay.js: enigImportAttachmentCallback: "+ctxt+"\n");
-
-  if (callbackArg.ipcBuffer.overflowed) {
-    WARNING_LOG("enigmailMessengerOverlay.js: enigImportAttachmentCallback: MESSAGE BUFFER OVERFLOW\n");
-  }
-
-  var dataLengthObj = new Object();
-  var dataText = callbackArg.ipcBuffer.getByteData(dataLengthObj);
-  callbackArg.ipcBuffer.shutdown();
-
-  var errorMsgObj= new Object();
-  var enigmailSvc =  GetEnigmailSvc();
-  var exitCode = enigmailSvc.importKey (window, 0, dataText, "", errorMsgObj);
-
-  if (exitCode == 0) {
-    EnigAlert(EnigGetString("successKeyImport")+"\n\n"+errorMsgObj.value);
-  }
-  else {
-    EnigError(EnigGetString("failKeyImport")+"\n\n"+errorMsgObj.value);
-  }
-
-}
 
 // handle double click events on Attachments
 function enigAttachmentListClick (elementId, event) {
@@ -1852,8 +1769,12 @@ function enigReceiveKeyTerminate (terminateArg, ipcRequest) {
     DEBUG_LOG("enigmailMessengerOverlay.js: enigReceiveKeyTerminate: exitCode = "+exitCode+"\n");
   }
 
+  var enigmailSvc = GetEnigmailSvc();
   if (exitCode==0) {
     enigMessageReload(false);
+    if (enigmailSvc) {
+      enigmailSvc.invalidateUserIdList();
+    }    
   }
   else {
     var errorMsg="";
@@ -1862,7 +1783,6 @@ function enigReceiveKeyTerminate (terminateArg, ipcRequest) {
 
       if (keygenConsole && keygenConsole.hasNewData()) {
         errorMsg = keygenConsole.getData();
-        var enigmailSvc = GetEnigmailSvc();
         if (enigmailSvc) {
           var statusFlagsObj=new Object();
           var statusMsgObj=new Object();
