@@ -11,30 +11,41 @@
  *
  * The Original Code is Enigmail.
  *
- * The Initial Developer of the Original Code is Patrick Brunschwig.
- * Portions created by Patrick Brunschwig <patrick.brunschwig@gmx.net> are
- * Copyright (C) 2004 Ramalingam Saravanan. All Rights Reserved.
+ * The Initial Developer of this code is Patrick Brunschwig.
+ * Portions created by Patrick Brunschwig <patrick.brunschwig@gmx.net>
+ * are Copyright (C) 2004 Patrick Brunschwig.
+ * All Rights Reserved.
  *
  * Contributor(s):
  *
  * Alternatively, the contents of this file may be used under the
- * terms of the GNU General Public License (the "GPL"), in which case
- * the provisions of the GPL are applicable instead of
+ * terms of the GNU General Public License (the "GPL") or the GNU 
+ * Lesser General Public License (the "LGPL"), in which case
+ * the provisions of the GPL or the LGPL are applicable instead of
  * those above. If you wish to allow use of your version of this
- * file only under the terms of the GPL and not to allow
+ * file only under the terms of the GPL or the LGPL and not to allow
  * others to use your version of this file under the MPL, indicate
  * your decision by deleting the provisions above and replace them
- * with the notice and other provisions required by the GPL.
+ * with the notice and other provisions required by the GPL or the 
+ * LGPL respectively.
  * If you do not delete the provisions above, a recipient
- * may use your version of this file under either the MPL or the
- * GPL.
+ * may use your version of this file under either the MPL, the
+ * GPL or the LGPL.
  */
 
+
 // helper functions for message composition
 
 function getRecipientsKeys(emailAddrs, forceSelection, matchedKeysObj, flagsObj) {
+  DEBUG_LOG("enigmailMsgComposeHelper.js: getRecipientsKeys: emailAddrs="+emailAddrs+"\n");
 
-  function getFlagVal(oldVal, newVal) {
+  function getFlagVal(oldVal, node, type, conflictObj) {
+    var newVal = Number(node.getAttribute(type));
+
+    if ((oldVal==2 && newVal==0) || (oldVal==0 && newVal==2)) {
+      conflictObj[type] = 1;
+    }
+    
     if (oldVal==0 || newVal==0) {
       return 0;
     }
@@ -47,11 +58,12 @@ function getRecipientsKeys(emailAddrs, forceSelection, matchedKeysObj, flagsObj)
   if (!enigmailSvc)
     return false;
     
-  flagsObj.value = -1;
+  flagsObj.value = 0;
   matchedKeysObj.value = "";
   var encrypt=1;
   var sign   =1;
   var pgpMime=1;
+  var conflicts = { sign: 0, encrypt: 0, pgpMime: 0};
   var addresses="{"+EnigStripEmail(emailAddrs.toLowerCase()).replace(/[, ]+/g, "}{")+"}";
   var keyList=new Array;
 
@@ -64,7 +76,7 @@ function getRecipientsKeys(emailAddrs, forceSelection, matchedKeysObj, flagsObj)
   
     if (rulesList.firstChild.nodeName=="parsererror") {
       EnigAlert("Invalid enigmail.xml file:\n"+ rulesList.firstChild.textContent);
-      return 0;
+      return true;
     }
     DEBUG_LOG("enigmailMsgComposeHelper.js: getRecipientsKeys: keys loaded\n");
     var node=rulesList.firstChild.firstChild;
@@ -78,9 +90,9 @@ function getRecipientsKeys(emailAddrs, forceSelection, matchedKeysObj, flagsObj)
               var email=addrList[addrIndex];
               var i=addresses.indexOf(email);
               while (i>=0) {
-                sign   =getFlagVal(sign,    node.getAttribute("sign"));
-                encrypt=getFlagVal(encrypt, node.getAttribute("encrypt"));
-                pgpMime=getFlagVal(pgpMime, node.getAttribute("pgpMime"));
+                sign   =getFlagVal(sign,    node, "sign", conflicts);
+                encrypt=getFlagVal(encrypt, node, "encrypt", conflicts);
+                pgpMime=getFlagVal(pgpMime, node, "pgpMime", conflicts);
     
                 // extract found address
                 var keyIds=node.getAttribute("keyId");
@@ -116,9 +128,11 @@ function getRecipientsKeys(emailAddrs, forceSelection, matchedKeysObj, flagsObj)
     var resultObj=new Object;
     for (i=0; i<addrList.length; i++) {
       if (addrList[i].length>0) {
-        if (foundAddresses.indexOf("{"+EnigStripEmail(addrList[i]).toLowerCase()+"}")==-1) {
-          inputObj.toAddress="{"+addrList[i]+"}";
+        var theAddr=EnigStripEmail(addrList[i]).toLowerCase();
+        if (foundAddresses.indexOf("{"+theAddr+"}")==-1) {
+          inputObj.toAddress="{"+theAddr+"}";
           inputObj.options="";
+          inputObj.command = "add";
           window.openDialog("chrome://enigmail/content/enigmailSingleRcptSettings.xul","", "dialog,modal,centerscreen,resizable", inputObj, resultObj);
           if (resultObj.cancelled==true) return false;
           sign   =getFlagVal(sign,    resultObj.sign);
@@ -143,8 +157,20 @@ function getRecipientsKeys(emailAddrs, forceSelection, matchedKeysObj, flagsObj)
     matchedKeysObj.value = keyList.join(", ");
     matchedKeysObj.value += addresses.replace(/\{/g, ", ").replace(/\}/g, "");
   }
-  flagsObj.value = sign | (encrypt << 2) | (pgpMime << 4);
+  flagsObj.sign = sign;
+  flagsObj.encrypt = encrypt;
+  flagsObj.pgpMime = pgpMime;
+  flagsObj.value = 1;
 
+  if ((!EnigGetPref("confirmBeforeSend")) && (conflicts.encrypt ||conflicts.sign)) {
+    var msg = "\n"+"- " + EnigGetString(sign ? "signYes" : "signNo");
+    msg += "\n"+"- " + EnigGetString(encrypt ? "encryptYes" : "encryptNo");
+    if (EnigGetPref("warnOnRulesConflict")==2) {
+      EnigSetPref("warnOnRulesConflict", 0);
+    }
+    if (!EnigConfirmPref(EnigGetString("rulesConflict", msg), "warnOnRulesConflict"))
+      return false;
+  }
   return true;
 }
 
