@@ -12,7 +12,6 @@ const EnigOutputWrap          = 32;
 const EnigOutputFormatFlowed  = 64;
 const EnigOutputCRLineBreak   = 512;
 const EnigOutputLFLineBreak   = 1024;
-const EnigDummyFile = "application/enigmail-dummy-file";
 
 const ENIG_ENIGMSGCOMPFIELDS_CONTRACTID = "@mozdev.org/enigmail/composefields;1";
 
@@ -24,7 +23,7 @@ var gEnigDirty, gEnigProcessed, gEnigTimeoutID;
 var gEnigSendPGPMime, gEnigModifiedAttach, gEnigSendMode;
 var gEnigSendModeDirty = 0;
 var gEnigNextCommand;
-var gEnigAccountId = null;
+var gEnigIdentity = null;
 
 if (typeof(GenericSendMessage)=="function") {
   // replace GenericSendMessage with our own version
@@ -98,17 +97,17 @@ function enigSetIdentityCallback(elementId) {
 
 function enigGetAccDefault(value) {
 
-  var enabled = gEnigAccountId.getBoolAttribute("enablePgp");
+  var enabled = gEnigIdentity.getBoolAttribute("enablePgp");
   if (value == "enabled")
     return enabled;
   if (enabled) {
     switch (value) {
     case 'encrypt':
-      return gEnigAccountId.getIntAttribute("defaultEncryptionPolicy");
+      return gEnigIdentity.getIntAttribute("defaultEncryptionPolicy");
     case 'signPlain':
-      return gEnigAccountId.getBoolAttribute("pgpSignPlain");
+      return gEnigIdentity.getBoolAttribute("pgpSignPlain");
     case 'signEnc':
-      return gEnigAccountId.getBoolAttribute("pgpSignEncrypted");
+      return gEnigIdentity.getBoolAttribute("pgpSignEncrypted");
     }
   }
   else {
@@ -124,10 +123,10 @@ function enigGetAccDefault(value) {
 }
 
 function enigSetIdentityDefaults() {
-  gEnigAccountId = getCurrentIdentity();
-  gEnigAccountId.getBoolAttribute("enablePgp");
+  gEnigIdentity = getCurrentIdentity();
+  gEnigIdentity.getBoolAttribute("enablePgp");
   if (enigGetAccDefault("enabled")) {
-    EnigGetSignMsg(gEnigAccountId);
+    EnigGetSignMsg(gEnigIdentity);
   }
 
   if (! gEnigSendModeDirty) {
@@ -197,7 +196,7 @@ function enigMsgComposeReset() {
   gEnigModifiedAttach=null;
   gEnigSendModeDirty = 0;
   gEnigSendMode = 0;
-  gEnigAccountId = null;
+  gEnigIdentity = null;
 
   EnigShowHeadersAll(true);
 
@@ -313,19 +312,10 @@ function enigUndoEncryption() {
       var node = bucketList.firstChild;
       while (node) {
         if (node.attachment.url == gEnigModifiedAttach[i].newUrl) {
-          if (gEnigModifiedAttach[i].dummyFile) {
-            node.attachment=null;
-            var delNode=node;
-            node=node.nextSibling;
-            bucketList.removeChild(delNode);
-            continue;
-          }
-          else {
-            node.attachment.url = gEnigModifiedAttach[i].origUrl;
-            node.attachment.name = gEnigModifiedAttach[i].origName;
-            node.attachment.temporary = gEnigModifiedAttach[i].origTemp;
-            node.attachment.contentType = gEnigModifiedAttach[i].origCType;
-          }
+          node.attachment.url = gEnigModifiedAttach[i].origUrl;
+          node.attachment.name = gEnigModifiedAttach[i].origName;
+          node.attachment.temporary = gEnigModifiedAttach[i].origTemp;
+          node.attachment.contentType = gEnigModifiedAttach[i].origCType;
           // delete encrypted file
           try {
             gEnigModifiedAttach[i].newFile.remove(false);
@@ -353,18 +343,15 @@ function enigReplaceEditorText(text) {
 
 function enigGoAccountManager()
 {
-    var server=null;
-    try {
-        var currentId=getCurrentIdentity();
-        var amService=Components.classes["@mozilla.org/messenger/account-manager;1"].getService();
-        var servers=amService.GetServersForIdentity(currentId);
-        var folderURI=servers.GetElementAt(0).QueryInterface(Components.interfaces.nsIMsgIncomingServer).serverURI;
-        server=GetMsgFolderFromUri(folderURI, true).server
-    } catch (ex) {}
-
-    window.openDialog("chrome://messenger/content/AccountManager.xul",
-                      "AccountManager", "chrome,modal,titlebar,resizable",
-                      { server: server, selectPage: 'am-enigprefs.xul' });
+  var server=null;
+  try {
+      var currentId=getCurrentIdentity();
+      var amService=Components.classes["@mozilla.org/messenger/account-manager;1"].getService();
+      var servers=amService.GetServersForIdentity(currentId);
+      var folderURI=servers.GetElementAt(0).QueryInterface(Components.interfaces.nsIMsgIncomingServer).serverURI;
+      server=GetMsgFolderFromUri(folderURI, true).server
+  } catch (ex) {}
+  window.openDialog("chrome://enigmail/content/am-enigprefs-edit.xul", "", "dialog,modal,centerscreen", {identity: currentId, account: server});
 }
 
 function enigDoPgpButton(what) {
@@ -374,9 +361,12 @@ function enigDoPgpButton(what) {
   try {
     if (!enigGetAccDefault("enabled")) {
       if (EnigConfirm(EnigGetString("configureNow"))) {
-          enigGoAccountManager();
+        enigGoAccountManager();
+        if (! gEnigIdentity.getBoolAttribute("enablePgp")) return;
+      }
+      else {
+        return;
       }
-      return;
     }
   }
   catch (ex) {}
@@ -519,25 +509,7 @@ function enigDisplaySignClickWarn() {
   if ((gEnigSendModeDirty<2) &&
       (enigGetAccDefault("signPlain") ||
        enigGetAccDefault("signEnc"))) {
-    try {
-      var warn=EnigGetPref("displaySignWarn");
-      if (warn) {
-          var checkValue = {value:false};
-          var buttonPressed = gEnigPromptSvc.confirmEx(window,
-                EnigGetString("warning"),
-                EnigGetString("signIconClicked"),
-                (gEnigPromptSvc.BUTTON_TITLE_OK * gEnigPromptSvc.BUTTON_POS_0),
-                null,null, null,
-                sComposeMsgsBundle.getString('CheckMsg'),
-                checkValue);
-          if (buttonPressed != 0) {
-            return;
-          }
-          if (checkValue.value) {
-            EnigSetPref("displaySignWarn", false);
-          }
-      }
-    } catch (ex) {}
+    EnigAlertPref(EnigGetString("signIconClicked"), "displaySignWarn");
   }
 }
 
@@ -657,9 +629,9 @@ function enigEncryptMsg(msgSendType) {
      var errorMsgObj    = new Object();
      gEnigModifiedAttach = null;
 
-     DEBUG_LOG("enigmailMsgComposeOverlay.js: enigEncryptMsg: currentId="+gEnigAccountId+
-              ", "+gEnigAccountId.email+"\n");
-     var fromAddr = gEnigAccountId.email;
+     DEBUG_LOG("enigmailMsgComposeOverlay.js: enigEncryptMsg: currentId="+gEnigIdentity+
+              ", "+gEnigIdentity.email+"\n");
+     var fromAddr = gEnigIdentity.email;
 
      var pgpEnabled = enigGetAccDefault("enabled");
 
@@ -686,8 +658,8 @@ function enigEncryptMsg(msgSendType) {
 
      sendFlags |= optSendFlags;
 
-     if (gEnigAccountId.getIntAttribute("pgpKeyMode")>0) {
-       var userIdValue = gEnigAccountId.getCharAttribute("pgpkeyId");
+     if (gEnigIdentity.getIntAttribute("pgpKeyMode")>0) {
+       var userIdValue = gEnigIdentity.getCharAttribute("pgpkeyId");
 
        if (!userIdValue) {
 
@@ -703,10 +675,10 @@ function enigEncryptMsg(msgSendType) {
 
        if (userIdValue) {
          fromAddr = userIdValue;
-         gEnigAccountId.setCharAttribute("pgpkeyId", userIdValue);
+         gEnigIdentity.setCharAttribute("pgpkeyId", userIdValue);
 
        } else {
-         gEnigAccountId.setIntAttribute("pgpKeyMode", 0);
+         gEnigIdentity.setIntAttribute("pgpKeyMode", 0);
        }
      }
 
@@ -735,7 +707,7 @@ function enigEncryptMsg(msgSendType) {
        var bccLC = EnigStripEmail(msgCompFields.bcc).toLowerCase()
        DEBUG_LOG("enigmailMsgComposeOverlay.js: enigEncryptMsg: BCC: "+bccLC+"\n");
 
-       var selfBCC = gEnigAccountId.email && (gEnigAccountId.email.toLowerCase() == bccLC);
+       var selfBCC = gEnigIdentity.email && (gEnigIdentity.email.toLowerCase() == bccLC);
 
        if (selfBCC) {
          DEBUG_LOG("enigmailMsgComposeOverlay.js: enigEncryptMsg: Self BCC\n");
@@ -792,7 +764,7 @@ function enigEncryptMsg(msgSendType) {
         var repeatSelection=0;
         while (repeatSelection<2) {
           if (perRecipientRules>0) {
-            var matchedKeysObj  =new Object;
+            var matchedKeysObj = new Object;
             var flagsObj=new Object;
             if (!getRecipientsKeys(toAddr, 
                                   (repeatSelection==1),
@@ -803,12 +775,8 @@ function enigEncryptMsg(msgSendType) {
             }
            
             if (matchedKeysObj.value) toAddr=matchedKeysObj.value;
-            if (flagsObj.value != -1) {
-              // sign:    0x0001 | 0x0002 => 3
-              // encrypt: 0x0004 | 0x0008 => 12
-              // pgpMime: 0x0010 | 0x0020 => 48
-              switch (flagsObj.value & 3) {
-               // sign
+            if (flagsObj.value) {
+              switch (flagsObj.sign) {
                case 0:
                  sendFlags &= ~ENIG_SIGN;
                  break;
@@ -817,8 +785,7 @@ function enigEncryptMsg(msgSendType) {
                  break;
               }
     
-              switch ((flagsObj.value & 12)>>2) {
-               // encrypt
+              switch (flagsObj.encrypt) {
                case 0:
                  sendFlags &= ~ENIG_ENCRYPT;
                  break;
@@ -827,8 +794,7 @@ function enigEncryptMsg(msgSendType) {
                  break;
               }
     
-              switch ((flagsObj.value & 48)>>4) {
-               // pgpMime
+              switch (flagsObj.pgpMime) {
                case 0:
                  sendFlags &= ~nsIEnigmail.SEND_PGP_MIME;
                  break;
@@ -930,9 +896,6 @@ function enigEncryptMsg(msgSendType) {
      if (sendFlags & nsIEnigmail.SAVE_MESSAGE) {
        // always enable PGP/MIME if message is saved
        sendFlags |= nsIEnigmail.SEND_PGP_MIME;
-       if (! hasAttachments) {
-         hasAttachments = enigCreateDummyAttachment();
-       }
      }
      
      if ( hasAttachments &&
@@ -1041,9 +1004,10 @@ function enigEncryptMsg(msgSendType) {
        try {
          var convert = DetermineConvertibility();
          if (convert == nsIMsgCompConvertible.No) {
-           if (!EnigConfirm(EnigGetString("strippingHTML")))
+           if (!EnigConfirm(EnigGetString("strippingHTML"))) {
              window.cancelSendMessage=true;
              return;
+           }
          }
        } catch (ex) {
        }
@@ -1230,16 +1194,6 @@ function enigEncryptMsg(msgSendType) {
          return;
        }
      } 
-     else if (isOffline &&
-                !EnigConfirm(EnigGetString("offlineNote")) ) {
-       // Abort send
-       if (gEnigProcessed)
-         enigUndoEncryption();
-         
-       window.cancelSendMessage=true;
-       return;
-
-     }
      else if ( (sendFlags & nsIEnigmail.SEND_WITH_CHECK) &&
                  !enigMessageSendCheck() ) {
        // Abort send
@@ -1311,7 +1265,7 @@ function enigGenericSendMessage( msgType )
 {
   dump("enigGenericSendMessage from XUL\n");
 
-  dump("Identity = " + gEnigAccountId + "\n");
+  dump("Identity = " + gEnigIdentity + "\n");
 
   if (gMsgCompose != null)
   {
@@ -1607,8 +1561,6 @@ function enigEncryptAttachments(bucketList, newAttachments, window, uiFlags,
     var newUri = ioServ.newFileURI(newFile);
     fileInfo.newUrl  = newUri.asciiSpec;
     fileInfo.newFile = newFile;
-    fileInfo.dummyFile = false;
-
 
     newAttachments.push(fileInfo);
     node = node.nextSibling;
@@ -1645,8 +1597,8 @@ function enigToggleAccountAttr(attrName)
 {
   DEBUG_LOG("enigmailMsgComposeOverlay.js: enigToggleAccountAttr('"+attrName+"')\n");
 
-  var oldValue = gEnigAccountId.getBoolAttribute(attrName);
-  gEnigAccountId.setBoolAttribute(attrName, !oldValue)
+  var oldValue = gEnigIdentity.getBoolAttribute(attrName);
+  gEnigIdentity.setBoolAttribute(attrName, !oldValue)
 
 }
 
@@ -1660,8 +1612,6 @@ function enigDecryptQuote(interactive) {
   if (!enigmailSvc)
     return;
 
-  enigmailRemoveDummyAttachment();
-
   var encoderFlags = EnigOutputFormatted | EnigOutputLFLineBreak;
 
   var docText = EnigEditorGetContentsAs("text/plain", encoderFlags);
@@ -1760,13 +1710,14 @@ function enigDecryptQuote(interactive) {
   var statusFlagsObj = new Object();
   var userIdObj      = new Object();
   var keyIdObj       = new Object();
+  var sigDateObj     = new Object();
   var errorMsgObj    = new Object();
 
   var uiFlags = nsIEnigmail.UI_UNVERIFIED_ENC_OK;
 
   var plainText = enigmailSvc.decryptMessage(window, uiFlags, cipherText,
                                  signatureObj, exitCodeObj, statusFlagsObj,
-                                 keyIdObj, userIdObj, errorMsgObj);
+                                 keyIdObj, userIdObj, sigDateObj, errorMsgObj);
 
   // Decode plaintext from charset to unicode
   plainText = EnigConvertToUnicode(plainText, charset);
@@ -1806,7 +1757,9 @@ function enigDecryptQuote(interactive) {
   }
 
   var doubleDashSeparator = EnigGetPref("doubleDashSeparator")
-  if (doubleDashSeparator) {
+  if (gMsgCompose.type != nsIMsgCompType.Template && 
+      gMsgCompose.type != nsIMsgCompType.Draft &&
+      doubleDashSeparator) {
     var signOffset = plainText.search(/[\r\n]-- +[\r\n]/);
 
     if (signOffset > 0) {
@@ -1866,7 +1819,7 @@ function enigDecryptQuote(interactive) {
   // Position cursor
   var replyOnTop = 1;
   try {
-    replyOnTop = gEnigAccountId.replyOnTop;
+    replyOnTop = gEnigIdentity.replyOnTop;
   } catch (ex) {}
 
   if (!indentStr || !quoteElement)
@@ -1942,8 +1895,16 @@ function enigGetChildOffset(parentNode, childNode) {
 }
 
 function EnigEditorInsertText(plainText) {
+  DEBUG_LOG("enigmailMsgComposeOverlay.js: EnigEditorInsertText\n");
   if (gEnigEditor) {
-    gEnigEditor.insertText(plainText);
+    var mailEditor;
+    try {
+      mailEditor = gEnigEditor.QueryInterface(Components.interfaces.nsIEditorMailSupport);
+      mailEditor.insertTextWithQuotations(plainText);
+    } catch (ex) {
+      DEBUG_LOG("enigmailMsgComposeOverlay.js: EnigEditorInsertText: no mail editor\n");
+      gEnigEditor.insertText(plainText);
+    }
   }
 }
 
@@ -1952,7 +1913,7 @@ function EnigEditorInsertAsQuotation(plainText) {
     var mailEditor;
     try {
       mailEditor = gEnigEditor.QueryInterface(Components.interfaces.nsIEditorMailSupport);
-    } catch (ex) {};
+    } catch (ex) {}
 
     if (!mailEditor)
       return null;
@@ -1972,10 +1933,12 @@ function EnigEditorSelectAll() {
 }
 
 function EnigEditorGetCharset() {
+  DEBUG_LOG("enigmailMsgComposeOverlay.js: EnigEditorGetCharset\n");
   return gEnigEditor.documentCharacterSet;
 }
 
 function EnigEditorGetContentsAs(mimeType, flags) {
+  DEBUG_LOG("enigmailMsgComposeOverlay.js: EnigEditorGetContentsAs\n");
   if (gEnigEditor) {
     return gEnigEditor.outputToString(mimeType, flags);
   }
@@ -2081,47 +2044,3 @@ EnigDocStateListener.prototype = {
   }
 }
   
-function enigCreateDummyAttachment() {
-  try {
-    // create dummy file
-    var fileAttach = Components.classes[ENIG_LOCAL_FILE_CONTRACTID].createInstance(Components.interfaces.nsILocalFile);
-    fileAttach.initWithPath(EnigGetTempDir());
-    fileAttach.append("sig.asc")
-    fileAttach.createUnique(Components.interfaces.NORMAL_FILE_TYPE, 0600);
-    
-    // attach file 
-    var ioService = Components.classes["@mozilla.org/network/io-service;1"]
-    ioService = ioService.getService(Components.interfaces.nsIIOService);
-    var fileHandler = ioService.getProtocolHandler("file").QueryInterface(Components.interfaces.nsIFileProtocolHandler);
-    var attachUrl = fileHandler.getURLSpecFromFile(fileAttach);
-    var attachment = Components.classes["@mozilla.org/messengercompose/attachment;1"].createInstance(Components.interfaces.nsIMsgAttachment);
-    attachment.url = attachUrl;
-    attachment.contentType=EnigDummyFile;
-    attachment.temporary = true;
-    AddAttachment(attachment);
-    
-    // make dummy file is deleted before exit
-    var extAppLauncher = Components.classes[ENIG_MIME_CONTRACTID].getService(Components.interfaces.nsPIExternalAppLauncher);
-    extAppLauncher.deleteTemporaryFileOnExit(fileAttach);
-    gEnigModifiedAttach = new Array();
-    gEnigModifiedAttach.push({'newUrl': attachUrl, 'dummyFile': true});
-  }
-  catch (ex) {
-    return false;
-  }
-  
-  return true;
-}
-
-function enigmailRemoveDummyAttachment() {
-  var bucketList = document.getElementById("attachmentBucket");
-  var hasAttachments = bucketList && bucketList.hasChildNodes();
-  
-  if (hasAttachments && bucketList.childNodes.length==1) {
-    if (bucketList.firstChild.attachment.contentType==EnigDummyFile) {
-      bucketList.firstChild.attachment=null;
-      bucketList.removeChild(bucketList.firstChild);
-    }
-  }
-}
-
