@@ -67,15 +67,25 @@ function enigStartHeaders()
   gEnigStatusBar.removeAttribute("encrypted");
 
   var enigmailBox = document.getElementById("expandedEnigmailBox");
+  var statusText = document.getElementById("enigmailStatusText");
 
   if (enigmailBox && !enigmailBox.collapsed) {
     enigmailBox.setAttribute("collapsed", "true");
 
-    var statusText = document.getElementById("expandedEnigmailStatusText");
-
-    if (statusText)
-      statusText.firstChild.data="*";
+    if (statusText) statusText.value="";
   }
+  
+  if (statusText)
+    statusText.setAttribute("class", "enigmailHeaderBoxLabelSignatureOk");
+
+  statusText = document.getElementById("collapsedEnigmailStatusText");
+
+  if (enigmailBox && !enigmailBox.collapsed) {
+    if (statusText) tatusText.value="";
+  }
+  
+  if (statusText)
+    statusText.setAttribute("class", "enigmailHeaderBoxLabelSignatureOk");
 
   var msgFrame = EnigGetFrame(window, "messagepane");
 
@@ -83,9 +93,7 @@ function enigStartHeaders()
     DEBUG_LOG("enigmailMsgHdrViewOverlay.js: msgFrame="+msgFrame+"\n");
 
     msgFrame.addEventListener("unload", enigMessageUnload, true);
-
-    if (EnigGetPref("autoDecrypt"))
-      msgFrame.addEventListener("load", enigMessageDecrypt, false);
+    msgFrame.addEventListener("load", enigMessageAutoDecrypt, false);
   }
 
   enigForgetEncryptedURI();
@@ -103,18 +111,62 @@ function enigEndHeaders()
   DEBUG_LOG("enigmailMsgHdrViewOverlay.js: enigEndHeaders\n");
   gEnigStatusBar.removeAttribute("signed");
   gEnigStatusBar.removeAttribute("encrypted");
+  var statusText = document.getElementById("enigmailStatusText");
+  if (statusText) {
+    statusText.setAttribute("class", "enigmailHeaderBoxLabelSignatureOk");
+  }
+  
+  statusText = document.getElementById("collapsedEnigmailStatusText");
+  if (statusText) {
+    statusText.setAttribute("class", "enigmailHeaderBoxLabelSignatureOk");
+  }
 }
 
 function enigBeforeStartHeaders() {
   return true;
 }
 
-function enigUpdateHdrIcons(exitCode, statusFlags, keyId, userId, errorMsg) {
+// Match the userId from gpg to the sender's from address
+function enigMatchUidToSender(userId) {
+  var fromAddr = currentHeaderData["from"].headerValue;
+  try {
+    fromAddr=EnigStripEmail(fromAddr);
+  }
+  catch(ex) {}
+  
+  var userIdList=userId.split(/\n/);
+  try {
+    for (var i=0; i<userIdList.length; i++) {
+      if (fromAddr.toLowerCase() == EnigStripEmail(userIdList[i]).toLowerCase()) {
+        userId = userIdList[i];
+        break;
+      }
+    }
+    if (i>=userIdList.length) userId=userIdList[0];
+  }
+  catch (ex) {
+    userId=userIdList[0];
+  }
+  return userId;
+}
+
+function enigUpdateHdrIcons(exitCode, statusFlags, keyId, userId, sigDetails, errorMsg) {
   DEBUG_LOG("enigmailMsgHdrViewOverlay.js: enigUpdateHdrIcons: exitCode="+exitCode+", statusFlags="+statusFlags+", keyId="+keyId+", userId="+userId+", "+errorMsg+"\n");
 
   gEnigLastEncryptedURI = GetLoadedMessage();
-  userId=EnigConvertGpgToUnicode(userId);
-  errorMsg=EnigConvertGpgToUnicode(errorMsg);
+  
+  if (userId && (userId.indexOf("\n")>=0)) {
+    var replaceUid=enigMatchUidToSender(userId);
+    errorMsg = errorMsg.replace(userId, replaceUid);
+    userId=EnigConvertGpgToUnicode(replaceUid);
+  }
+
+  if (exitCode == ENIG_POSSIBLE_PGPMIME) {
+    exitCode = 0;
+  }
+  else {
+    errorMsg=EnigConvertGpgToUnicode(errorMsg);
+  }
   
   var errorLines="";
   var fullStatusInfo=errorMsg;
@@ -134,6 +186,7 @@ function enigUpdateHdrIcons(exitCode, statusFlags, keyId, userId, errorMsg) {
 
   var statusInfo = "";
   var statusLine = "";
+  var statusArr = [];
 
   if (statusFlags & nsIEnigmail.NODATA) {
     if (statusFlags & nsIEnigmail.PGP_MIME_SIGNED)
@@ -160,65 +213,89 @@ function enigUpdateHdrIcons(exitCode, statusFlags, keyId, userId, errorMsg) {
         !(statusFlags & (nsIEnigmail.UNVERIFIED_SIGNATURE |
           nsIEnigmail.IMPORTED_KEY))) {
     // Normal exit / display message
-    statusInfo = errorMsg;
     statusLine = errorMsg;
-
-  } else if (keyId) {
-    statusInfo = EnigGetString("keyNeeded",keyId);
-
-    if (statusFlags & nsIEnigmail.INLINE_KEY) {
-      statusLine = statusInfo + EnigGetString("clickDecrypt");
-    } else {
-      statusLine = statusInfo + EnigGetString("clickPen");
+    statusInfo = statusLine;
+    
+    if (sigDetails) {
+      var detailArr=sigDetails.split(/ /);
+      var dat=new Date(detailArr[2]*1000);
+      var appLocale = Components.classes[ENIG_LOCALE_SVC_CONTRACTID].getService(Components.interfaces.nsILocaleService).getApplicationLocale();
+      var dateFormat = Components.classes[ENIG_DATE_FORMAT_CONTRACTID].getService(Components.interfaces.nsIScriptableDateFormat);
+      dateTime = dateFormat.FormatDateTime(appLocale.getCategory("NSILOCALE_TIME"), 
+                          dateFormat.dateFormatShort,
+                          dateFormat.timeFormatNoSeconds,
+                          dat.getFullYear(), dat.getMonth()+1, dat.getDate(),
+                          dat.getHours(), dat.getMinutes(), 0);
+      var txt = EnigGetString("keyAndSigDate", detailArr[0].substr(-8, 8), dateTime);
+      statusArr.push(txt);
+      statusInfo += "\n" + txt;
+      // 933D 9948 DC0F A861 471B 10A9 D8A8 07C7 CCEC 227B
+      // 4628 4358 0136 4A06 B9EA FE9D 3C5D F224
+      var fpr = detailArr[0].match(/^(....)(....)(....)(....)(....)(....)(....)(....)(....)?(....)?$/);
+      if (fpr && fpr.length > 2) {
+        fpr.shift()
+        statusInfo += "\n"+EnigGetString("keyFpr",fpr.join(" "));
+      }
     }
-
-    statusInfo = EnigGetString("unverifiedSig");
-    statusLine = statusInfo + EnigGetString("clickPen");
-    statusInfo += "\n\n" + errorMsg;
-
-  } else if (statusFlags & nsIEnigmail.UNVERIFIED_SIGNATURE) {
-    statusInfo = EnigGetString("unverifiedSig");
-    statusLine = statusInfo + EnigGetString("clickQueryPenDetails");
-    statusInfo += "\n\n" + errorMsg;
-
-  } else if (statusFlags & (nsIEnigmail.BAD_SIGNATURE |
-                            nsIEnigmail.UNVERIFIED_SIGNATURE |
-                            nsIEnigmail.EXPIRED_SIGNATURE |
-                            nsIEnigmail.EXPIRED_KEY_SIGNATURE)) {
-    statusInfo = EnigGetString("failedSig");
-    statusLine = statusInfo + EnigGetString("clickPenDetails");
-    statusInfo += "\n\n" + errorMsg;
-
-  } else if (statusFlags & nsIEnigmail.DECRYPTION_INCOMPLETE) {
-    statusInfo = EnigGetString("incompleteDecrypt");
-    statusLine = statusInfo + EnigGetString("clickKey");
-    statusInfo += "\n\n" + errorMsg;
-
-  } else if (statusFlags & nsIEnigmail.DECRYPTION_FAILED) {
-    if (statusFlags & nsIEnigmail.NO_SECKEY) {
-      statusInfo = EnigGetString("needKey");
-    } else {
-      statusInfo = EnigGetString("failedDecrypt");
-    }
-
-    statusLine = statusInfo + EnigGetString("clickKeyDetails");
-    statusInfo += "\n\n" + errorMsg;
-
-  } else if (statusFlags & nsIEnigmail.BAD_PASSPHRASE) {
-    statusInfo = EnigGetString("badPhrase");
-    statusLine = statusInfo + EnigGetString("clickDecryptRetry");
-    statusInfo += "\n\n" + errorMsg;
-
-  } else if (statusFlags & nsIEnigmail.IMPORTED_KEY) {
-    EnigAlert(errorMsg);
+    fullStatusInfo = statusInfo;
 
   } else {
-    statusInfo = EnigGetString("failedDecryptVerify");
-    statusLine = statusInfo + EnigGetString("viewInfo");
-    statusInfo += "\n\n" + errorMsg;
+    if (keyId) {
+      statusInfo = EnigGetString("keyNeeded",keyId);
+  
+      if (statusFlags & nsIEnigmail.INLINE_KEY) {
+        statusLine = statusInfo + EnigGetString("clickDecrypt");
+      } else {
+        statusLine = statusInfo + EnigGetString("clickPen");
+      }
+  
+      statusInfo = EnigGetString("unverifiedSig");
+      statusLine = statusInfo + EnigGetString("clickPen");
+      statusInfo += "\n\n" + errorMsg;
+  
+    } else if (statusFlags & nsIEnigmail.UNVERIFIED_SIGNATURE) {
+      statusInfo = EnigGetString("unverifiedSig");
+      statusLine = statusInfo + EnigGetString("clickQueryPenDetails");
+      statusInfo += "\n\n" + errorMsg;
+      
+    } else if (statusFlags & (nsIEnigmail.BAD_SIGNATURE |
+                              nsIEnigmail.UNVERIFIED_SIGNATURE |
+                              nsIEnigmail.EXPIRED_SIGNATURE |
+                              nsIEnigmail.EXPIRED_KEY_SIGNATURE)) {
+      statusInfo = EnigGetString("failedSig");
+      statusLine = statusInfo + EnigGetString("clickPenDetails");
+      statusInfo += "\n\n" + errorMsg;
+  
+    } else if (statusFlags & nsIEnigmail.DECRYPTION_INCOMPLETE) {
+      statusInfo = EnigGetString("incompleteDecrypt");
+      statusLine = statusInfo + EnigGetString("clickKey");
+      statusInfo += "\n\n" + errorMsg;
+  
+    } else if (statusFlags & nsIEnigmail.DECRYPTION_FAILED) {
+      if (statusFlags & nsIEnigmail.NO_SECKEY) {
+        statusInfo = EnigGetString("needKey");
+      } else {
+        statusInfo = EnigGetString("failedDecrypt");
+      }
+  
+      statusLine = statusInfo + EnigGetString("clickKeyDetails");
+      statusInfo += "\n\n" + errorMsg;
+  
+    } else if (statusFlags & nsIEnigmail.BAD_PASSPHRASE) {
+      statusInfo = EnigGetString("badPhrase");
+      statusLine = statusInfo + EnigGetString("clickDecryptRetry");
+      statusInfo += "\n\n" + errorMsg;
+  
+    } else if (statusFlags & nsIEnigmail.IMPORTED_KEY) {
+      EnigAlert(errorMsg);
+  
+    } else {
+      statusInfo = EnigGetString("failedDecryptVerify");
+      statusLine = statusInfo + EnigGetString("viewInfo");
+      statusInfo += "\n\n" + errorMsg;
+    }
   }
 
-
   if (statusFlags & nsIEnigmail.DECRYPTION_OKAY ||
       (gEnigStatusBar.getAttribute("encrypted")=="ok")) {
     if (!statusInfo) {
@@ -256,21 +333,34 @@ function enigUpdateHdrIcons(exitCode, statusFlags, keyId, userId, errorMsg) {
                         keyId: keyId,
                         userId: userId,
                         statusLine: statusLine,
+                        msgSigned: msgSigned,
+                        statusArr: statusArr,
                         statusInfo: statusInfo,
                         fullStatusInfo: fullStatusInfo };
 
   var enigmailBox = document.getElementById("expandedEnigmailBox");
-  var statusText  = document.getElementById("expandedEnigmailStatusText");
+  var statusText  = document.getElementById("enigmailStatusText");
+  var expStatusText  = document.getElementById("expandedEnigmailStatusText");
 
-  if (statusLine) {
-    statusText.firstChild.data= statusLine +" ";
-    enigmailBox.removeAttribute("collapsed");
-
-  } else {
-    statusText.firstChild.data= "*";
-    enigmailBox.setAttribute("collapsed", "true");
+  if (statusArr.length>0) {
+    expStatusText.firstChild.data = statusArr[0];
+    expStatusText.setAttribute("state", "true");
   }
-
+  else {
+    expStatusText.firstChild.data = "*";
+    expStatusText.setAttribute("state", "false");
+  }
+  
+  if (statusLine) {
+    statusText.value = statusLine +" ";
+    enigmailBox.removeAttribute("collapsed");
+    enigDisplayExtendedStatus(true);
+  } else {
+    statusText.value = "";
+    enigmailBox.setAttribute("collapsed", "true");
+    enigDisplayExtendedStatus(false);
+  }
+  
   if (!gSMIMEContainer)
     return;
 
@@ -284,6 +374,7 @@ function enigUpdateHdrIcons(exitCode, statusFlags, keyId, userId, errorMsg) {
       // Display untrusted/bad signature icon
       gSignedUINode.setAttribute("signed", "notok");
       statusText.setAttribute("class", "enigmailHeaderBoxLabelSignatureNotOk");
+      expStatusText.setAttribute("class", "enigmailHeaderBoxLabelSignatureNotOk");
       gEnigStatusBar.setAttribute("signed", "notok");
     }
     else if ((statusFlags & nsIEnigmail.GOOD_SIGNATURE) &&
@@ -294,6 +385,7 @@ function enigUpdateHdrIcons(exitCode, statusFlags, keyId, userId, errorMsg) {
       // Display trusted good signature icon
       gSignedUINode.setAttribute("signed", "ok");
       statusText.setAttribute("class", "enigmailHeaderBoxLabelSignatureOk");
+      expStatusText.setAttribute("class", "enigmailHeaderBoxLabelSignatureOk");
       gEnigStatusBar.setAttribute("signed", "ok");
     }
     else if (statusFlags & (nsIEnigmail.UNVERIFIED_SIGNATURE |
@@ -304,12 +396,9 @@ function enigUpdateHdrIcons(exitCode, statusFlags, keyId, userId, errorMsg) {
       // Display unverified signature icon
       gSignedUINode.setAttribute("signed", "unknown");
       statusText.setAttribute("class", "enigmailHeaderBoxLabelSignatureUnknown");
+      expStatusText.setAttribute("class", "enigmailHeaderBoxLabelSignatureUnknown");
       gEnigStatusBar.setAttribute("signed", "unknown");
     }
-    else {
-    //  gEnigStatusBar.removeAttribute("signed");
-    //  statusText.setAttribute("class", "enigmailHeaderBoxLabelSignatureOk");
-    }
 
     if (statusFlags & nsIEnigmail.DECRYPTION_OKAY) {
       var enigMimeService = Components.classes[ENIG_ENIGMIMESERVICE_CONTRACTID].getService(Components.interfaces.nsIEnigMimeService);
@@ -328,15 +417,12 @@ function enigUpdateHdrIcons(exitCode, statusFlags, keyId, userId, errorMsg) {
       gEncryptedUINode.setAttribute("encrypted", "notok");
       gEnigStatusBar.setAttribute("encrypted", "notok");
     }
-    else {
-      // gEnigStatusBar.removeAttribute("encrypted");
-    }
 
   } catch (ex) {}
 }
 
 function enigDispSecurityContext() {
-  var optList = ["pgpSecurityInfo", "copySecurityInfo", "showPhoto"];
+  var optList = ["pgpSecurityInfo", "copySecurityInfo", "showPhoto", "signKey", "editKeyTrust"];
   for (var j=0; j<optList.length; j++) {
     var menuElement = document.getElementById("enigmail_"+optList[j]);
     if (gEnigSecurityInfo) {
@@ -351,9 +437,29 @@ function enigDispSecurityContext() {
     if (! (gEnigSecurityInfo.statusFlags & nsIEnigmail.PHOTO_AVAILABLE)) {
       document.getElementById("enigmail_showPhoto").setAttribute("disabled", "true");
     }
+    if (! (gEnigSecurityInfo.msgSigned &&  !(gEnigSecurityInfo.statusFlags & nsIEnigmail.UNVERIFIED_SIGNATURE))) {
+      document.getElementById("enigmail_signKey").setAttribute("disabled", "true");
+      document.getElementById("enigmail_editKeyTrust").setAttribute("disabled", "true");
+    }
   }
 }
 
+function enigEditKeyTrust() {
+  var inputObj = {
+    keyId: gEnigSecurityInfo.keyId
+  }
+  window.openDialog("chrome://enigmail/content/enigmailEditKeyTrustDlg.xul","", "dialog,modal,centerscreen", inputObj);
+}
+
+function enigSignKey() {
+  var inputObj = {
+    keyId: gEnigSecurityInfo.keyId,
+    userId: "0xCCEC227B"
+  }
+  window.openDialog("chrome://enigmail/content/enigmailSignKeyDlg.xul","", "dialog,modal,centerscreen", inputObj);
+
+}
+
 function enigMsgHdrViewLoad(event)
 {
   DEBUG_LOG("enigmailMsgHdrViewOverlay.js: enigMsgHdrViewLoad\n");
@@ -421,6 +527,13 @@ function enigShowPhoto() {
   }
 }
 
+function enigCreateRuleFromAddress(emailAddressNode) {
+  if (emailAddressNode)
+  {
+    EnigNewRule(emailAddressNode.getAttribute("emailAddress"));
+  }
+}
+
 function enigForgetEncryptedURI()
 {
   if (gEnigLastEncryptedURI)
@@ -437,6 +550,7 @@ function enigMsgHdrViewHide() {
   DEBUG_LOG("enigmailMsgHdrViewOverlay.js: enigMsgHdrViewHide\n");
   var enigmailBox = document.getElementById("expandedEnigmailBox");
   enigmailBox.collapsed=true;
+
   gEnigSecurityInfo = { statusFlags: 0,
                       keyId: "",
                       userId: "",
@@ -455,6 +569,40 @@ function enigMsgHdrViewUnhide() {
   }
 }
 
+function enigDisplayExtendedStatus(displayMe) {
+  var expStatusText  = document.getElementById("expandedEnigmailStatusText");
+  if (displayMe && expStatusText.getAttribute("state") == "true") {
+    if (expStatusText.getAttribute("display") == "true") {
+      expStatusText.removeAttribute("collapsed");
+    }
+    else {
+      expStatusText.setAttribute("collapsed", "true");
+    }
+  }
+  else {
+    expStatusText.setAttribute("collapsed", "true");
+  }
+}
+
+function enigToggleHeaderView() {
+  var viewToggle = document.getElementById("enigToggleHeaderView");
+  var expandedText = document.getElementById("expandedEnigmailStatusText");
+  var state = viewToggle.getAttribute("state");
+  
+  if (state=="true") {
+    viewToggle.setAttribute("state", "false");
+    viewToggle.setAttribute("class", "collapsedHeaderViewButton");
+    expandedText.setAttribute("display", "false");
+    enigDisplayExtendedStatus(false);
+  }
+  else {
+    viewToggle.setAttribute("state", "true");
+    viewToggle.setAttribute("class", "expandHeaderViewButton");
+    expandedText.setAttribute("display", "true");
+    enigDisplayExtendedStatus(true);
+  }
+}
+
 
 addEventListener('messagepane-loaded', enigMsgHdrViewLoad, true);
 addEventListener('messagepane-hide', enigMsgHdrViewHide, true);
@@ -602,7 +750,7 @@ EnigMimeHeaderSink.prototype =
     throw Components.results.NS_NOINTERFACE;
   },
 
-  updateSecurityStatus: function(uriSpec, exitCode, statusFlags, keyId, userId, errorMsg)
+  updateSecurityStatus: function(uriSpec, exitCode, statusFlags, keyId, userId, sigDetails, errorMsg)
   {
     DEBUG_LOG("enigmailMsgHdrViewOverlay.js: EnigMimeHeaderSink.updateSecurityStatus: uriSpec="+uriSpec+"\n");
 
@@ -611,7 +759,7 @@ EnigMimeHeaderSink.prototype =
     DEBUG_LOG("enigmailMsgHdrViewOverlay.js: EnigMimeHeaderSink.updateSecurityStatus: msgUriSpec="+msgUriSpec+"\n");
 
     if (!uriSpec || (uriSpec == msgUriSpec)) {
-      enigUpdateHdrIcons(exitCode, statusFlags, keyId, userId, errorMsg);
+      enigUpdateHdrIcons(exitCode, statusFlags, keyId, userId, sigDetails, errorMsg);
     }
 
     return;
